;;; -*- Mode: LISP; Package: USER; Base: 10 -*-
;;; $Header: /ct/debug/dianatags.l,v 1.13 84/08/13 13:30:33 bill Exp $
(putprop 'dianatags "$Revision: 1.13 $" 'rcs_revision)
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                  ;;;
;;;                              dianatags.l                         ;;;
;;;                                                                  ;;;
;;; William Brew                                           9-6-83    ;;;
;;;                                                                  ;;;
;;; Tags package for the Ada debugger. Allows easy positioning of    ;;;
;;; the code window etc.                                             ;;;
;;;                                                                  ;;;
;;; This file is part of a proprietary software project.  Source     ;;;
;;; code and documentation describing implementation details are     ;;;
;;; available on a confidential, non-disclosure basis only.  These   ;;;
;;; materials, including this file in particular, are trade secrets  ;;;
;;; of Computer * Thought Corporation.                               ;;;
;;;                                                                  ;;;
;;; (c) Copyright 1982 and 1983,  Computer * Thought Corporation.    ;;;
;;;     All Rights Reserved.                                         ;;;
;;;                                                                  ;;;
;;; Reference materials:                                             ;;;
;;;   Foderaro and Sklower, The FRANZ LISP Manual, September 1981.   ;;;
;;;   Weinreb and Moon, LISP MACHINE MANUAL, Symbolics, July 1981.   ;;;
;;;   Charniak et al., 1980.  Artificial Intelligence Programming.   ;;;
;;;   Miller, 1982.  The C*T Ada Tutor: Guide to the Implementation. ;;;
;;;                                                                  ;;;
;;;   The following code assumes familiarity with these materials.   ;;;
;;;                                                                  ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Ensure presence of needed files. 

(comment Assumes ct_load and some suitable file_map are present)

(eval-when (compile load eval) (ct_load 'aip))	   ;AIP macros pkg. 

(eval-when (compile load eval) (ct_load 'compat))  ;Franz/LM compat pkg.

(eval-when (compile load eval) (ct_load 'ctstrl))  ;New strings

(eval-when (compile load eval) (ct_load 'ctio))    ;Compatable io

(eval-when (compile load eval) (ct_load 'ctflav))  ;Flavors


(eval-when (compile load eval) (ct_load 'dbutils)) ;Debugger utilities


#+franz
(eval-when (compile load eval) (ct_load 'screens)) ;Window flavs, asks
#+lispm
(eval-when (compile load eval) (ct_load 'lmscreens));Window flavs, asks


(eval-when (compile load eval) (ct_load 'ferec))   ;Interpretter records

(eval-when (compile load eval) (ct_load 'diana))   ;Diana utilities

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Compiler Declarations and Global Variables -- 

#+franz (declare (macros t))
#+franz (setq *flavor-expand-macros* t)

; Specials used.

(declare (special *db%hidden_files*))

#+franz
(declare (localf db%display_node db%get_tag_node db%get_tag_pname
	     db%get_tag_description db%display))

(defconst *db%top_window* 0.1
  "The fraction of the window above the cursor when we show the top of something.")

(defconst *db%bottom_window* 0.9
  "The fraction of the window above the cursor when we show the bottom of something.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Macro definitions --

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Flavor definitions --

; 
; Tags are used by the debugger to "mark" regions of the souce text. The
; region is given by the lx_srcpos of a diana node. Tags may have various
; operations associated with them depending on their usage. (pc marker,
; break point etc.) The basic operations include printing their name and
; description, and displaying themselves in a scroll window.

; 
; The basic flavor for a tag.
; 

(ct_defflavor db%vanilla_tag_flavor
    ((pname nil)		       ; A string which is this tag's pname
     (description nil)	               ; A string which describes this tag
     (node nil)		               ; The diana node (or function hook)
     (origin 'center)	               ; The "origin" for displaying this tag 
     (over_origin 'top)                ; The origin if the tag is too big
    )
    ()
    #+lispm :initable-instance-variables
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Externally Callable Functions/Methods -- 

; 
; Initialize the tags. 
; 

(defun db%init_dianatags ()
    nil)

; 
; Startup the tags. 
; 

(defun db%start_dianatags ()
    nil)

; 
; The following methods impliment the basic properties of the tag
; abstraction.
; 

; 
; Get the node for a tag.
; 

(ct_defmethod (db%vanilla_tag_flavor node) ()
    (db%get_tag_node node))

; 
; Write the node instance variable for a tag.
; 

(ct_defmethod (db%vanilla_tag_flavor set-node) (val)
    (setq node val))

; 
; Print the name of the tag on the indicated stream. (ala format)
; 

(ct_defmethod (db%vanilla_tag_flavor printself) (stream)
    (let ((tag_pname (db%get_tag_pname pname node)))
	(ct_if stream
	    (ct_princ tag_pname stream)
	    tag_pname)))

; 
; Print a description of the tag on the indicated stream. (ala format)
; 

(ct_defmethod (db%vanilla_tag_flavor describeself) (stream)
    (let ((tag_description (db%get_tag_description description node)))
	(ct_if stream
	    (ct_princ tag_description stream)
	    tag_description)))

; 
; A method to display a tag in a scroll window. First check that we have a
; good diana node. If so then display it.
; 

(ct_defmethod (db%vanilla_tag_flavor displayself) (window stream)
    (let ((the_node (db%get_tag_node node))
	  (the_name (db%get_tag_pname pname node)))
	(cond (the_node
	       (db%display_node the_name window the_node origin over_origin)
	       t)
	      (t (ct_format stream "The tag ~a is unbound.~%" the_name) nil))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Internal Use Only Functions/Methods -- 

; 
; Display the source region corresponding to a node in the indicated window.
; Origin is where we want the text displayed (top center or bottom).
; Over_origin is used if the region is too big for the window.
; First get the region from the node. If we want top or bottom origin then
; do the obvious thing. If we are centering then check the region size
; against the window size. If it fits then do the obvious thing. If not,
; then look at over_origin to resolve the problem.
; 

(defun db%display_node (name window node origin over_origin)
    (let* ((srcpos (diana_get node 'lx_srcpos))
	   (start (source_region%startchar srcpos))
	   (end (source_region%endchar srcpos)))
	(ct_selectq (and srcpos origin)
	    (top (db%display name window node start *db%top_window*))
	    (center (ct_if (> (get-iv db%debug_window window 'lines-displayed)
			      (- (source_region%linenumber srcpos)
				 (source_region%linstart srcpos)))
			(db%display name window node (!/ (+ start end) 2) ':center)
			(ct_selectq over_origin
			    (top (db%display name window node start *db%top_window*))
			    (bottom (db%display name window node end *db%bottom_window*))
			    (otherwise (lose 'db%dt_origin 'db%display_node
					   '("Unknown origin value"))))))
	    (bottom (db%display name window node end *db%bottom_window*))
	    (otherwise (db%message "Not displayable.")))))

; 
; Try getting the node for a tag. If node is a diana node, return it. Otherwise
; try a hook function. If still no luck return nil.
; 

(defun db%get_tag_node (node)
    (let (new_node)
	(cond ((diana_nodep node) node)
	      ((diana_nodep (setq new_node (db%maybe_run_hook node))) new_node))))

; 
; Try to get the print name of a tag. First check the pname var. If nothing
; there then try asking the node.
; 

(defun db%get_tag_pname (pname node)
    (ct_if pname
	(db%coerce_to_string pname)
	(db%diana_printself (db%get_tag_node node) nil)))

; 
; Try to get the description of a tag. First check the description var. If nothing
; there then try asking the node.
; 

(defun db%get_tag_description (description node)
    (ct_if description
	(db%coerce_to_string description)
	(db%diana_describeself (db%get_tag_node node) nil)))

; 
; A function  which tells a scroll window to display a node with the indicated
; character at the indicated origin. Check for the special case of hidden files.
; This prevents us from breaking on "wazzoo" or other nasties. Also, check for
; dn_compilations and eliminate them also because they don't really belong to
; a particalar file even though the interpeter gives them a source region.
; 

(defun db%display (name window node ochar origin)
    (ct_if (or (member (source_region%path (diana_get node 'lx_srcpos)) *db%hidden_files*)
	       (eq (diana_nodetype_get node) 'dn_compilation))
	(db%message
	    "The tag ~a is in an unviewable file and cannot be displayed." name)
	(db%source_to_window window (db%node_to_sroot node) ochar origin)))

;;; eof
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
